__title__ = 'aioimap'
__version__ = '0.2.7'
__summary__ = 'Receive e-mails from an IMAP server asynchronously and trigger a callback with the message.'
__uri__ = 'https://github.com/surajiyer/aioimap'
__author__ = 'Suraj Iyer'
__email__ = 'me@surajiyer.com'
__license__ = 'MIT'